/*
 *	MakeANote.c
 *	MakeANote() - Make a Csound note statement and stick it in a file if the
 *	probability is high enough
 */
#include "CB.h"
#include <math.h>
#include <stdio.h>

extern float	*gFrequency, *gDeviation, *gDensity, *gAmplitude, 
		*gPanning, *gWaveform, *gDuration;
extern double	gLengthGrain;
long	controlCycleNumber;	/* is equal to 1/10 millisecond */
extern Boolean gMakingNotes, gDurFreq;
extern short	gScoreVRefNum, gScoreDFileNum;
 
int
MakeANote()
{
	float duration, frequency, deviation, density, amplitude,
		panning, waveform, probability, timeSecs, durSecs;
	unsigned short randShort;
	char string[200];
	long size;
	Str255 pascalStr;
	OSErr error;
	
	if(controlCycleNumber == 0)
	{
		sprintf((char *)pascalStr, "f1 0 8192 10 1");
		c2pstr((char *)pascalStr);
		ShowNote(pascalStr);
		sprintf(string, "f1 0 8192 10 1");
		c2pstr(string);
		string[0]++;
		string[string[0]] = 0xd;
		size = string[0];
		error = FSWrite(gScoreDFileNum, &size, string+1);
	}
	/* Find density and duration to determine probability */
	density = InterpFunctionValue(gDensity, controlCycleNumber);
	if(gDurFreq)
	{
		frequency = InterpFunctionValue(gFrequency, controlCycleNumber);
		duration = 1000.0/frequency;
	}
	else
		duration = InterpFunctionValue(gDuration, controlCycleNumber);
	timeSecs = controlCycleNumber * 0.001;
	
	if(timeSecs >= gLengthGrain)
	{
		gMakingNotes = FALSE;
		DisposPtr((Ptr)gFrequency);
		DisposPtr((Ptr)gDeviation);
		DisposPtr((Ptr)gDensity);
		DisposPtr((Ptr)gPanning);
		DisposPtr((Ptr)gAmplitude);
		DisposPtr((Ptr)gWaveform);
		DisposPtr((Ptr)gDuration);
		sprintf((char *)pascalStr, "Finished");
		c2pstr((char *)pascalStr);
		ShowNote(pascalStr);
		return(0);
		FSClose(gScoreDFileNum);
	}

	/* 
	 * Density can vary from 0 to 100, Duration can vary from 1 to 100 mSec.
	 * A duration of 2 mSec would require a 500 notes a second to achieve
	 * 100% density. This would be 1 note every 2 mSecs (0.002 secs) or .5 notes every
	 * cycle, with our cycle of 0.001 seconds. We double this as Curtis Roads 
	 * recommends.
	 */
	
	probability = (density * 0.04) / duration;
	randShort = Random();
	if((float)randShort < (65536.0 * probability))
	{
		frequency = InterpFunctionValue(gFrequency, controlCycleNumber);
		deviation = InterpFunctionValue(gDeviation, controlCycleNumber);
		amplitude = InterpFunctionValue(gAmplitude, controlCycleNumber);
		panning = InterpFunctionValue(gPanning, controlCycleNumber);
		waveform = InterpFunctionValue(gWaveform, controlCycleNumber);
			
		randShort = Random();
		frequency += ((2 * deviation * (float)randShort) / 65536.0) - deviation;
		
		durSecs = duration * 0.001;
		
		sprintf((char *)pascalStr, "i1 %f %f %f %f %f %f", timeSecs, durSecs, frequency, amplitude, panning, waveform);
		c2pstr((char *)pascalStr);
		ShowNote(pascalStr);
		sprintf(string, "i1 %f %f %f %f %f %f", timeSecs, durSecs, frequency, amplitude, panning, waveform);
		c2pstr(string);
		string[0]++;
		string[string[0]] = 0xd;
		size = string[0];
		error = FSWrite(gScoreDFileNum, &size, string+1);
	}
	controlCycleNumber++;
}
